package com.example.nto.service.impl;


import com.example.nto.aspect.annotation.LogExample;
import com.example.nto.entity.Employee;
import com.example.nto.exception.EmployeeNotFoundException;
import com.example.nto.exception.SomethingWentWrongException;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {
    // Класс отвечает за основную логику приложения. Тут мы получаем из бд и обработав их возвращаем клиенту.

    private final EmployeeRepository employeeRepository;   // Это объект для работы с бд.

    @Override
    @LogExample
    public Employee get(String login) {
        final String error_message = "Работник с логином " + login + " не найден!";

        try {
            return employeeRepository.findAll().stream()
                    .filter(e -> Objects.equals(e.getLogin(), login)).findFirst()
                    .orElseThrow(() -> new EmployeeNotFoundException(error_message));
        }
        catch (EmployeeNotFoundException e) { throw new EmployeeNotFoundException(e.getMessage()); }
        catch (Exception e) { throw new SomethingWentWrongException("Непредвиденная ошибка"); }
    }

    @Override
    @LogExample
    public Employee update(Employee employee, LocalDateTime last_visit) {
        final String error_message = "Работник с логином " + employee.getLogin() + " не найден!";

        try {
            // Проверка, что работник существует.
            employeeRepository.findById(employee.getId()).orElseThrow(() -> new EmployeeNotFoundException(error_message));

            employee.setLastVisit(last_visit);
            employeeRepository.save(employee);
            return employee;
        }
        catch (EmployeeNotFoundException e) { throw new EmployeeNotFoundException(e.getMessage()); }
        catch (Exception e) { throw new SomethingWentWrongException("Непредвиденная ошибка"); }
    }
}
